package es.uca.xmleye.prefs;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import java.util.*;
import junit.framework.*;
import junit.textui.*;
import java.io.File;

import es.uca.xmleye.prefs.ExcepcionPreferencias;
import es.uca.xmleye.prefs.GestorPreferenciasSerializado;

/** 
    Pruebas de unidad de {@link GestorPreferenciasSerializado}.
*/
public class PruebaGestorPreferenciasSerializado extends TestCase 
{

  /** 
      Crea una nueva colección de casos de prueba.
  */
  public PruebaGestorPreferenciasSerializado(String nombre) 
  {
    super(nombre);
  }

  /** 
      Crea el conjunto de casos de prueba para JUnit.
  */
  public static Test suite() 
  {
    return new TestSuite(PruebaGestorPreferenciasSerializado.class);
  }

  /** Ejecuta las pruebas. */
  public static void main(String[] args) 
  {
    TestRunner.run(suite());
  }

  /***** PRUEBAS ************************************/

  private final static String FICHERO_PREFERENCIAS_TMP = "_unittestprefs";

  /** 
      <p>Probamos el guardado de un valor escalar. Se hace de la
      siguiente forma:</p>

      <ol>
        <li>abrimos un nuevo fichero</li>
	<li>guardamos un valor escalar bajo una clave</li>
	<li>cerramos el fichero</li>
	<li>volvemos a abrirlo</li>
	<li>leemos el valor, que tiene que ser el mismo</li>
      </ol>
  */
  public void testGuardarEscalar() {
    try {
      final String clave   = "$foo";
      final String valor = "barra";
      
      GestorPreferenciasSerializado prefs = new GestorPreferenciasSerializado(FICHERO_PREFERENCIAS_TMP);
      assertTrue(prefs.getPrefEscalar(clave) ==  null);
      prefs.cargar();
      prefs.setPrefEscalar(clave, valor);
      prefs.guardar();
      prefs = null;
      
      prefs = new GestorPreferenciasSerializado(FICHERO_PREFERENCIAS_TMP);
      prefs.cargar();
      assertTrue(valor.equals(prefs.getPrefEscalar(clave)));
      prefs.guardar();
      prefs = null;
    } 
    catch (ExcepcionPreferencias prefEx) {
      fail(prefEx.toString());
    }
  }

  /** 
      <p>Probamos el guardado de un valor de tipo lista. El
      procedimiento es análogo al de testGuardarEscalar.</p>

      @see #testGuardarEscalar()
  */
  public void testGuardarLista() {
    try {
      final String claveListaNoVacia = "@elems-lista";
      ArrayList<String> listaNoVacia = new ArrayList<String>();
      listaNoVacia.add("uno");
      listaNoVacia.add("dos");
      listaNoVacia.add("tres");
      
      final String claveListaVacia = "@empty-lista";
      ArrayList<String> listaVacia = new ArrayList<String>();
      
      GestorPreferenciasSerializado prefs = new GestorPreferenciasSerializado(FICHERO_PREFERENCIAS_TMP);
      assertTrue(prefs.getPrefLista(claveListaNoVacia) == null);
      assertTrue(prefs.getPrefLista(claveListaVacia) == null);
      prefs.cargar();
      prefs.setPrefLista(claveListaNoVacia, listaNoVacia);
      prefs.setPrefLista(claveListaVacia, listaVacia);
      prefs.guardar();
      prefs = null;
      
      prefs  = new GestorPreferenciasSerializado(FICHERO_PREFERENCIAS_TMP);
      prefs.cargar();
      assertTrue(listaNoVacia.equals(prefs.getPrefLista(claveListaNoVacia)));
      assertTrue(listaVacia.equals(prefs.getPrefLista(claveListaVacia)));
      prefs.guardar();
      prefs = null;
    }
    catch (ExcepcionPreferencias pEx) {
      fail(pEx.toString());
    }
  }

  /** Prepara los recursos necesarios para las pruebas. */
  protected void setUp()  {}

  /** Libera los recursos usados en las pruebas. */ 
  protected void tearDown() 
  {
    // Borramos el fichero temporal usado
    new File(FICHERO_PREFERENCIAS_TMP).delete();
  }

}
